"""Constants for the SOMweb lib."""
import logging
import re

LOGGER = logging.getLogger(__package__)

SOMWEB_URI_TEMPLATE = "https://{}.somweb.world"

RE_DOORS = re.compile(r'<\s*input\s+type\s*=\s*"submit"\s+class\s*=\s*"tab-door[\s\w-]*"\s+name\s*=\s*"tab-door\d+"\s+id\s*=\s*"tab-door(?P<id>\d+)"\s+value="(?P<name>[\w\s]+)"\s*\/?>', re.MULTILINE)
RE_WEBTOKEN = re.compile(r'<\s*input\s+id\s*=\s*"webtoken".*value="(?P<webtoken>\w+)"\/>', re.MULTILINE)



from logging import exception
import requests
import json
from enum import Enum

from .const import LOGGER, RE_WEBTOKEN, RE_DOORS, SOMWEB_URI_TEMPLATE

#
# TODO: Requests should include timeouts and also handle excpetions gracefully - see https://requests.readthedocs.io/en/master/user/quickstart/#timeouts
#

class DoorStatusType(Enum):
    Open = 1
    Closed = 2
    Unknown = 3

class DoorActionType(Enum):
    Close = 0
    Open = 1

class Door:
    def __init__(self, id, name):
        self.id = id
        self.name = name

class DoorStatus:
    def __init__(self, id, status):
        self.id = id
        self.status = status

class SomwebClient:
    __somWebPageContent = None
    __currentToken = None
    __username = None
    __password = None
    __doors = None
    __req = requests.Session()

    __door_status_types = {
        "OK": DoorStatusType.Closed,
        "FAIL": DoorStatusType.Open,
    }

    def __init__(self, somWebUDI, username, password):
        """
        Initialize SOMweb authenticator
        """
        self.__base_url = SOMWEB_URI_TEMPLATE.format(somWebUDI)
        self.__udi = somWebUDI
        self.__username = username
        self.__password = password

    @property
    def udi(self):
        return self.__udi

    @udi.setter
    def udi(self, value):
        raise exception("UDI cannot be set")

    @udi.deleter
    def udi(self):
        raise exception("UDI cannot be deleted")

    def authenticate(self):
        form_data = {
            "login": self.__username,
            "pass": self.__password,
            "send-login": "Sign in",
        }

        response = self.__req.post(self.__base_url + "/index.php", form_data)
        if not response.ok:
            LOGGER.error("Authentication failed. Reason: %s", response.reason)
            return False

        self.__somWebPageContent = response.text
        self.__currentToken = self.__extractWebToken(self.__somWebPageContent)

        return bool(self.__currentToken)

    def getDoorStatus(self, doorId):
        status = 1  # 1 = closed and 0 = open - SOMweb returns "OK" if sent status equals actual status or "FAIL" if status is the opposite
        bit = 0  # When set to 1 seems to define that 1 is to be returned if sent status matches actual status - if they don't match return is always FALSE
        url = f"/isg/statusDoor.php?numdoor={doorId}&status={status}&bit={bit}"
        response = self.__req.get(self.__base_url + url)
        if not response.ok:
            LOGGER.error("Failed getting door status. Reason: %s", response.reason)
            exception("Failed getting door status. Reason: %s", response.reason)

        return self.__door_status_types.get(response.text, DoorStatusType.Unknown)

    def getDoors(self):
        if not self.__doors:
            matches = RE_DOORS.finditer(self.__somWebPageContent)
            self.__doors = list(map(lambda m: Door(m.group("id"), m.group("name")), matches))

        return self.__doors

    def getAllDoorStatuses(self):
        doorIds = list(map(lambda d: d.id, self.getDoors()))
        url = f"/isg/statusDoorAll.php?webtoken={self.__currentToken}"
        response = self.__req.get(self.__base_url + url)
        data = json.loads(response.text)
        self.__currentToken = data["11"]

        #statuses = {k: data[k] for k in doorIds}

        # SOMweb returns string on closed ("0") and int on open (1)
        door_state_switcher = {
            "0": DoorStatusType.Closed,
            1: DoorStatusType.Open,
        }

        return list(map(lambda id: DoorStatus(id, door_state_switcher.get(data[id], DoorStatusType.Unknown)), doorIds))

    def openDoor(self, doorId):
        return self.doorAction(doorId, DoorActionType.Open)

    def closeDoor(self, doorId):
        return self.doorAction(doorId, DoorActionType.Close)

    def doorAction(self, doorId, doorAction):
        door_action_to_status_switcher = {
            DoorActionType.Close: DoorStatusType.Closed,
            DoorActionType.Open: DoorStatusType.Open,
        }
        requestedDoorStatus = door_action_to_status_switcher.get(doorAction)

        return self.getDoorStatus(doorId) == requestedDoorStatus or self.toogleDoorPosition(doorId)

    def toogleDoorPosition(self, doorId):
        url = f"/isg/opendoor.php?numdoor={doorId}&status=0&webtoken={self.__currentToken}"
        response = self.__req.get(self.__base_url + url)
        return response.ok and "OK" == response.text

    def __extractWebToken(self, html_content):
        """Parse web token from SOMweb HTML"""
        match = RE_WEBTOKEN.search(html_content)
        return None if match is None else match.group("webtoken")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Inspired by https://github.com/navdeep-G/setup.py

# Note: To use the 'upload' functionality of this file, you must:
#   $ pipenv install twine --dev

import io
import os
import sys
from shutil import Error, rmtree

from setuptools import find_packages, setup, Command

# Package meta-data.
NAME = "somweb"
DESCRIPTION = "SOMweb client. Open/close Garage doors produced by SOMMER (base+/pro+/tiga/tiga+/barrier systems)"
URL = "https://github.com/taarskog/pysomweb"
EMAIL = "somweb@heiigjen.com"
AUTHOR = "Trond Aarskog"
REQUIRES_PYTHON = ">=3.6.0"
VERSION = None  # Version taken from package

# What packages are required for this module to be executed?
# with io.open(os.path.join(here, "requirements.txt"), encoding="utf-8") as f:
#     REQUIRED = f.readlines()

REQUIRED = [
    'requests',
]

# What packages are optional?
EXTRAS = {
    # 'fancy feature': ['django'],
}

with open("LICENSE") as f:
    license = f.read()

# The rest you shouldn't have to touch too much :)
# ------------------------------------------------
# Except, perhaps the License and Trove Classifiers!
# If you do change the License, remember to change the Trove Classifier for that!

here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
try:
    with io.open(os.path.join(here, "README.md"), encoding="utf-8") as f:
        long_description = "\n" + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

# Load the package's __version__.py module as a dictionary.
about = {}
if not VERSION:
    project_slug = NAME.lower().replace("-", "_").replace(" ", "_")
    with open(os.path.join(here, project_slug, "__version__.py")) as f:
        exec(f.read(), about)
else:
    #about["__version__"] = VERSION
    raise Error("Required file missing. Package should have a __VERSION__ file!")

class UploadCommand(Command):
    """Support setup.py upload."""

    description = "Build and publish the package."
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print("\033[1m{0}\033[0m".format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status("Removing previous builds…")
            rmtree(os.path.join(here, "dist"))
        except OSError:
            pass

        self.status("Building Source and Wheel distribution…")
        os.system(f"{sys.executable} setup.py sdist bdist_wheel")

        self.status("Uploading the package to PyPI via Twine…")
        os.system("twine upload dist/*")

        self.status("Pushing git tags…")
        os.system("git tag v{0}".format(about["__version__"]))
        os.system("git push --tags")

        sys.exit()


# Where the magic happens:
setup(
    name=NAME,
    version=about["__version__"],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    download_url=f"{URL}/archive/v{about['__version__']}.tar.gz",
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    keywords=[
        "sommer",
        "SOMweb",
        "garage door",
        "home assistant",
        "home automation",
        "heiigjen",
    ],
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    license="MIT",
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        "Development Status :: 3 - Alpha",  # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
        "Topic :: Home Automation",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    # $ setup.py publish support.
    cmdclass={
        "upload": UploadCommand,
    },
)
